/* Dateiname:    lcd.c
 * Author:       Jochen Rurlnder
 * Version:      1.0
 * Datum:        26.10.2003
 * Homepage:     http://www.rurlaender.net (AVR - Libraries)
 * Beschreibung:
 * Diese Datei ist zu Steuerung des TWI (I2C) Buses gedacht.
 */
#include <avr/twi.h>

#define CLOCK	4000000 //Systemtakt 4MHz

void init_i2c(void)
{
	/* Author:       Jochen Rurlnder
	 * Version:      1.0
	 * Datum  :      26.10.2003
	 * Beschreibung:
	 * Initialisieren des TWI Busese. Berechnet wird mit der eingetragenen
	 * Prozessor Frequenz (CLOCK)
	 */
	TWBR = (CLOCK / 100000UL - 16) / 2;
}

int start_i2c(unsigned char address)
{
	/* Author:       Jochen Rurlnder
	 * Version:      1.0
	 * Datum  :      26.10.2003
	 * Beschreibung:
	 * Senden der IC Startbedingung + Addresse des anzusprechenden Gerts
	 * Returnwerte:
	 * 0 -> Startbedinung erfolgreich gesendet
	 * 1 -> Probleme beim sender der Start-Bedinung
	 * 2 -> Probleme beim senden der Addresse
	 *      bzw. kein Device mit der angegebenen Addresse
	 */
	 */
	//Senden der START Bedingung
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);

	//Warten bis TWI Bedingung gesendet wurde
	while(!(TWCR & (1<<TWINT)));

	//TWI Status berprfen
	if((TWSR & 0xF8) != TW_START) return 1;

	//Adresse des anzusprechenden Devices senden
	TWDR = address;
	TWCR = (1<<TWINT) | (1<<TWEN);

	//Warten bis TWI Bedingung gesendet wurde
	while(!(TWCR & (1<<TWINT)));

	//TWI Status berprfen
	if((TWSR & 0xF8) != TW_MT_SLA_ACK) return 2;

	return 0;
}

int rep_start_i2c(unsigned char address)
{
	/* Author:       Jochen Rurlnder
	 * Version:      1.0
	 * Datum  :      26.10.2003
	 * Beschreibung:
	 * Senden der IC Repeated-Startbedingung + Addresse des anzusprechenden Gerts
	 * Returnwerte:
	 * 0 -> Startbedinung erfolgreich gesendet
	 * 1 -> Probleme beim sender der Start-Bedinung
	 * 2 -> Probleme beim senden der Addresse
	 *      bzw. kein Device mit der angegebenen Addresse
	 */
	//Senden der START Kondition
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);

	//Warten bis TWI Bedingung gesendet wurde
	while(!(TWCR & (1<<TWINT)));

	//TWI Status berprfen
	if((TWSR & 0xF8) != TW_REP_START) return 1;

	//Adresse des anzusprechenden Devices senden
	TWDR = address;
	TWCR = (1<<TWINT) | (1<<TWEN);

	//Warten bis TWI Bedingung gesendet wurde
	while(!(TWCR & (1<<TWINT)));

	//TWI Status berprfen
	if((TWSR & 0xF8) != TW_MR_SLA_ACK) return 2;

	return 0;
}

void stop_i2c(void)
{
	/* Author:       Jochen Rurlnder
	 * Version:      1.0
	 * Datum  :      26.10.2003
	 * Beschreibung:
	 * Senden der IC Stopbedingung
	 */
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
}

void write_i2c(unsigned char data)
{
	/* Author:       Jochen Rurlnder
	 * Version:      1.0
	 * Datum  :      26.10.2003
	 * Beschreibung:
	 * Daten an das addressierte Gert schicken
	 */
	//Daten an das addressierte Devices senden
	TWDR = data;
	TWCR = (1<<TWINT) | (1<<TWEN);

	//Warten bis TWI Bedingung gesendet wurde
	while(!(TWCR & (1<<TWINT)));
}

unsigned char read_i2c_ack(void)
{
	/* Author:       Jochen Rurlnder
	 * Version:      1.0
	 * Datum  :      26.10.2003
	 * Beschreibung:
	 * Daten vom addressierten Gert lesen und mit ACK besttigen
	 * Fr alle Bytes bis auf das letzte zu bertragende Byte
	 */
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);

	//Warten bis TWI Bedingung gesendet wurde
	while(!(TWCR & (1<<TWINT)));

	//TWI Status berprfen
	if((TWSR & 0xF8) == TW_MR_DATA_ACK) return TWDR;
	return 1;
}

unsigned char read_i2c_nack(void)
{
	/* Author:       Jochen Rurlnder
	 * Version:      1.0
	 * Datum  :      26.10.2003
	 * Beschreibung:
	 * Fr das letzte zu bertragende Byte
	 */
	TWCR = (1<<TWINT) | (1<<TWEN);

	//Warten bis TWI Bedingung gesendet wurde
	while(!(TWCR & (1<<TWINT)));

	//TWI Status berprfen
	if((TWSR & 0xF8) == TW_MR_DATA_NACK) return TWDR;
	return 2;
}
